@@ -13,7 +13,6 @@ import os.path
 import argparse
 import subprocess
 import re
-import subprocess
 from shutil import which
 from colorama import init, Fore, Style
 
@@ -27,14 +26,6 @@ from . import compilers
 home = os.path.normpath(os.path.dirname(sys.argv[0]) + "/..")
 languages = ["ca", "en", "es", "fr", "de"]
 
-cxx = "g++"
-cxxflags = " -std=c++11 -D_JUDGE_ -O2 -DNDEBUG -Wall -Wextra -Wno-sign-compare -Wshadow "
-cxxflags_fallback = " -D_JUDGE_ -O2 -DNDEBUG "
-
-cc = "gcc"
-ccflags = " -D_JUDGE_ -O2 -DNDEBUG -Wall -Wextra -Wno-sign-compare "
-ccflags_fallback = ""
-
 errors = []
 
 
@@ -95,6 +86,23 @@ def make_executable():
     print()
     return com
 
+def make_executable_rec():
+    """Makes the executable files for the problem in the cwd"""
+
+    handler = sorted(glob.glob("handler.yml"))
+    if handler:
+        make_executable()
+    else:
+        for d in next(os.walk('.'))[1]:
+            if d in languages:
+                print(Style.DIM + os.getcwd() + ' ' + d + Style.RESET_ALL)
+                os.chdir(d)
+                make_executable()
+                os.chdir("..")
+            else:
+                print(Style.DIM + "skipping " + d + Style.RESET_ALL)
+            print()
+
 # ----------------------------------------------------------------------------
 # Make correct files
 # ----------------------------------------------------------------------------
@@ -116,8 +124,37 @@ def make_corrects(com, iterations=1):
         tst = os.path.splitext(inp)[0]
         time = com.execute(tst, True, iterations)
 
+        if handler["handler"] == "graphic":
+            os.rename("output.png", tst + ".cor")
         outsize = os.path.getsize(tst + ".cor")
-        print(Style.DIM + '\t\ttime: %f\t\tsize: %s' % (time, util.convert_bytes(outsize)) + Style.RESET_ALL)
+
+        try:
+            if 'Run' not in handler['compilers']:
+                print()
+        except Exception:
+            print()
+
+        print(Style.DIM + 'time: %f\t\tsize: %s' % (time, util.convert_bytes(outsize)) + Style.RESET_ALL)
+
+
+def make_corrects_rec():
+    """Makes the correct files for the problem in the cwd"""
+
+    handler = sorted(glob.glob("handler.yml"))
+    if handler:
+        com = make_executable()
+        make_corrects(com)
+    else:
+        for d in next(os.walk('.'))[1]:
+            if d in languages:
+                print(Style.DIM + os.getcwd() + ' ' + d + Style.RESET_ALL)
+                os.chdir(d)
+                com = make_executable()
+                make_corrects(com)
+                os.chdir("..")
+            else:
+                print(Style.DIM + "skipping " + d + Style.RESET_ALL)
+            print()
 
 # ----------------------------------------------------------------------------
 # Verify program
@@ -130,7 +167,7 @@ def verify_program(program, correct_extension='', iterations=1):
     if not util.file_exists("handler.yml"):
         raise Exception("handler.yml does not exist")
     handler = util.read_yml("handler.yml")
-    if handler["handler"] != "std":
+    if handler["handler"] != "std" and handler["handler"] != "graphic":
         raise Exception("unknown handler")
 
     # compile
@@ -158,8 +195,8 @@ def verify_program(program, correct_extension='', iterations=1):
             available_list.append([solution, com])
 
     print()
-    unsupported_list = [x for x in solution_list if x not in [
-        y[0] for y in available_list]]
+    unsupported_list = [x for x in solution_list if (x not in [
+        y[0] for y in available_list] and x[-1] != '~' and not x.endswith('bak'))]
     if unsupported_list != []:
         print(Fore.YELLOW + "NOTICE: The following solutions are still not supported and will NOT be verified: ", end='')
         for elem in unsupported_list:
@@ -179,6 +216,8 @@ def verify_program(program, correct_extension='', iterations=1):
         for test in tests:
             tst = os.path.splitext(test)[0]
             time = compiler.execute(tst, False, iterations)
+            if handler["handler"] == "graphic":
+                os.rename("output.png", tst + ext + ".out")
             outsize = os.path.getsize(tst + ext + ".out")
 
             r = subprocess.call(["cmp", tst + ext + ".out", tst + ".cor"])
@@ -189,7 +228,7 @@ def verify_program(program, correct_extension='', iterations=1):
                 msg = "OK"
                 util.del_file(tst + ext + ".out")
             print((Fore.GREEN if msg == 'OK' else Fore.RED) +
-                  "%s.inp:\t\t%s\t\ttime: %f\t\tsize: %s" %
+                  "%s.inp:\t\t%s\ntime: %f\t\tsize: %s" %
                   (tst, msg, time, util.convert_bytes(outsize)) + Style.RESET_ALL)
             if outsize > 2000000 and not os.path.isfile(tst + ext + ".ops"):
                 print(Fore.YELLOW + 'Warning: The output file is bigger than 2MB, you may need a .ops file for this test case.')
@@ -207,7 +246,7 @@ def clean_files(forced=False):
     removed_list = []
     for dirpath, dirnames, filenames in os.walk('.'):
         for filename in filenames:
-            if re.match('.*\.exe|.*\.cor|problem\..*\.pdf|problem\..*\.ps', filename):
+            if re.match('.*\.exe|.*\.cor|problem\..*\.pdf|problem\..*\.ps|a\.out', filename):
                 removed_list.append(dirpath + '/' + filename)
 
     if removed_list == []:
@@ -261,7 +300,13 @@ def make_prints_3(lang, ori):
             size = subprocess.getoutput(
                 "identify -format '(%%w$\\\\times$%%h)' %s.cor" % jj)
             graphic = "[%s]" % size
-            os.system("convert %s.cor %s.cor.eps" % (jj, jj))
+            r = os.system("convert %s.cor %s.cor.eps" % (jj, jj))
+            if r != 0:
+                print(Fore.RED + "ImageMagick error!", end = '')
+                if r == 256:
+                    print(" You must change it's security policy to be able to convert the images to EPS.", end = '')
+                print(Style.RESET_ALL)
+                sys.exit(0)
 
         sample2 += r"\SampleTwoColInputOutput%s{%s}{%s}" % (graphic, jj, num)
         sample1 += r"\SampleOneColInputOutput%s{%s}{%s}" % (graphic, jj, num)
@@ -294,8 +339,8 @@ handler.yml: \verbatimtabinput{handler.yml}
     """ % (lang, sample2, sample1, lang, lang, src, dat, lang, lang, scores)
 
     util.write_file("main.tex", t)
-
     print(Style.BRIGHT + "Generating .pdf file...   ", end=Style.RESET_ALL)
+    sys.stdout.flush()
     r = os.system("latex -interaction scrollmode main > main.err")
     if r != 0:
         os.system('cat main.err')
@@ -351,6 +396,7 @@ def make_prints():
                 os.chdir("..")
             else:
                 print(Style.DIM + "skipping " + d + Style.RESET_ALL)
+            print()
 
 
 # ----------------------------------------------------------------------------
@@ -378,6 +424,7 @@ def make_all(iterations=1):
                       os.getcwd() + "..." + Style.RESET_ALL)
                 print()
                 com = make_executable()
+                print()
                 make_corrects(com)
                 print()
                 verify_program("solution", com.extension(), iterations)
@@ -418,8 +465,8 @@ def make_recursive_2():
         if util.file_exists("solution.cc") or util.file_exists("solution.hs"):
             try:
                 if 1:
-                    make_executable()
-                    make_corrects()
+                    com = make_executable()
+                    make_corrects(com)
                     make_prints()
             except Exception as e:
                 print("\a")
@@ -495,14 +542,14 @@ def main():
     # Create and configure the option parser
     parser = argparse.ArgumentParser(
         usage="%(prog)s [options] [paths]",
-        description="Make different tasks in problem directories.",
+        description="Make different tasks in problem directories. If no arguments are specified, the script will compile the programs and generate the correct output of the problems. Then, it will check if all the possible solutions in different languages are correct. Finally it'll generate the .pdf files of the problem.",
     )
 
     parser.add_argument("--executable", help="make executable in the cwd",
                         action="store_true")
     parser.add_argument("--corrects", help="make correct files in the cwd",
                         action="store_true")
-    parser.add_argument("--prints", help="make printable files in the cwd",
+    parser.add_argument("--pdf", help="make printable files in the cwd",
                         action="store_true")
     parser.add_argument("--all", help="make executable, correct and printable files in the cwd (default)",
                         action="store_true")
@@ -522,17 +569,17 @@ def main():
                         action="store_true", default=False)
 
     # Parse options with real arguments
-    args, paths = parser.parse_known_args()
+    args = parser.parse_args()
 
     # Do the work
     done = False
     if args.executable:
         done = True
-        make_executable()
+        make_executable_rec()
     if args.corrects:
         done = True
-        make_corrects()
-    if args.prints:
+        make_corrects_rec()
+    if args.pdf:
         done = True
         make_prints()
     if args.all:
@@ -540,28 +587,21 @@ def main():
         make_all(args.iterations)
     if args.recursive:
         done = True
-        if paths == []:
-            paths = (".",)
-        make_recursive(paths)
+        make_recursive(".")
     if args.list:
         done = True
-        if paths == []:
-            paths = (".",)
-        make_list(paths)
+        make_list(".")
     if args.verify:
         done = True
-        if args.verify == None:
-            verify_program("solution", iterations=args.iterations)
-        else:
-            for d in next(os.walk('.'))[1]:
-                os.chdir(d)
-                if args.verify in ' '.join(glob.glob('*')):
-                    print(Style.BRIGHT + "Working on " +
-                          os.getcwd() + "..." + Style.RESET_ALL)
-                    print()
-                    verify_program(args.verify, iterations=args.iterations)
-                os.chdir('..')
-
+        verify_program(args.verify, iterations=args.iterations)
+        for d in next(os.walk('.'))[1]:
+            os.chdir(d)
+            if args.verify in ' '.join(glob.glob('*')):
+                print(Style.BRIGHT + "Working on " +
+                      os.getcwd() + "..." + Style.RESET_ALL)
+                print()
+                verify_program(args.verify, iterations=args.iterations)
+            os.chdir('..')
     if args.clean:
         done = True
         if args.force:
